@@ -1,7 +1,3 @@
-#ifndef lint
-static char *RCSid() { return RCSid("$Id: eval.c,v 1.119.2.1 2014/12/05 18:09:33 sfeam Exp $"); }
-#endif
-
 /* GNUPLOT - eval.c */
 
 /*[
@@ -34,33 +30,34 @@ static char *RCSid() { return RCSid("$Id: eval.c,v 1.119.2.1 2014/12/05 18:09:33
  * to the extent permitted by applicable law.
 ]*/
 
-/* HBB 20010724: I moved several variables and functions from parse.c
- * to here, because they're involved with *evaluating* functions, not
- * with parsing them: evaluate_at(), fpe(), and fpe_env */
-
 #include "eval.h"
 
 #include "syscfg.h"
 #include "alloc.h"
+#include "amos_airy.h"	/* Airy functions from AMOS */
+#include "complexfun.h"
 #include "datafile.h"
 #include "datablock.h"
 #include "external.h"	/* for f_calle */
 #include "internal.h"
 #include "libcerf.h"
+#include "misc.h"	/* for called_from() */
 #include "specfun.h"
 #include "standard.h"
 #include "util.h"
 #include "version.h"
 #include "term_api.h"
+#include "voxelgrid.h"
 
 #include <signal.h>
 #include <setjmp.h>
 
 /* Internal prototypes */
-static RETSIGTYPE fpe __PROTO((int an_int));
+static RETSIGTYPE fpe(int an_int);
 
 /* Global variables exported by this module */
-struct udvt_entry udv_pi = { NULL, "pi", FALSE, {INTGR, {0} } };
+struct udvt_entry udv_pi = { NULL, "pi", {INTGR, {0} } };
+struct udvt_entry *udv_I;
 struct udvt_entry *udv_NaN;
 /* first in linked list */
 struct udvt_entry *first_udv = &udv_pi;
@@ -68,8 +65,23 @@ struct udft_entry *first_udf = NULL;
 /* pointer to first udv users can delete */
 struct udvt_entry **udv_user_head;
 
+/* Various abnormal conditions during evaluation of an action table
+ * (the stored form of an expression) are signalled by setting
+ * undefined = TRUE.
+ * NB:  A test for  "if (undefined)"  is only valid immediately
+ * following a call to evaluate_at() or eval_link_function().
+ */
 TBOOLEAN undefined;
 
+enum int64_overflow overflow_handling = INT64_OVERFLOW_TO_FLOAT;
+
+/* Normally an error such as an undefined variable encountered
+ * during expression evaluation causes an exit via int_error().
+ * In some contexts such a hard failure is undesireable.
+ * Setting this to TRUE causes such evaluations to return NaN instead.
+ */
+TBOOLEAN eval_fail_soft = FALSE;
+
 /* The stack this operates on */
 static struct value stack[STACK_DEPTH];
 static int s_p = -1;		/* stack pointer */
@@ -79,7 +91,7 @@ static int jump_offset;		/* to be modified by 'jump' operators */
 
 /* The table of built-in functions */
 /* These must strictly parallel enum operators in eval.h */
-const struct ft_entry GPFAR ft[] =
+const struct ft_entry ft[] =
 {
     /* internal functions: */
     {"push",  f_push},
@@ -94,6 +106,7 @@ const struct ft_entry GPFAR ft[] =
     {"lnot",  f_lnot},
     {"bnot",  f_bnot},
     {"uminus",  f_uminus},
+    {"nop",  f_nop},
     {"lor",  f_lor},
     {"land",  f_land},
     {"bor",  f_bor},
@@ -119,8 +132,11 @@ const struct ft_entry GPFAR ft[] =
     {"concatenate",  f_concatenate},	/* for string variables only */
     {"eqs",  f_eqs},			/* for string variables only */
     {"nes",  f_nes},			/* for string variables only */
-    {"[]",  f_range},			/* for string variables only */
+    {"[]",  f_range},			/* substring or array slice */
+    {"[]",  f_index},			/* for array variables only */
+    {"||",  f_cardinality},		/* for array variables only */
     {"assign", f_assign},		/* assignment operator '=' */
+    {"eval", f_eval},			/* function block */
     {"jump",  f_jump},
     {"jumpz",  f_jumpz},
     {"jumpnz",  f_jumpnz},
@@ -138,6 +154,7 @@ const struct ft_entry GPFAR ft[] =
     {"stringcolumn",  f_stringcolumn},	/* for using specs */
     {"strcol",  f_stringcolumn},	/* shorthand form */
     {"columnhead",  f_columnhead},
+    {"columnheader",  f_columnhead},
     {"valid",  f_valid},
     {"timecolumn",  f_timecolumn},
 
@@ -145,6 +162,7 @@ const struct ft_entry GPFAR ft[] =
     {"real",  f_real},
     {"imag",  f_imag},
     {"arg",  f_arg},
+    {"conj",  f_conjg},
     {"conjg",  f_conjg},
     {"sin",  f_sin},
     {"cos",  f_cos},
@@ -160,60 +178,103 @@ const struct ft_entry GPFAR ft[] =
     {"EllipticE",  f_ellip_second},
     {"EllipticPi", f_ellip_third},
     {"int",  f_int},
+    {"round",  f_round},
     {"abs",  f_abs},
     {"sgn",  f_sgn},
     {"sqrt",  f_sqrt},
+    {"cbrt",  f_cbrt},
     {"exp",  f_exp},
     {"log10",  f_log10},
     {"log",  f_log},
+    {"besi0",  f_besi0},
+    {"besi1",  f_besi1},
+    {"besin",  f_besin},
     {"besj0",  f_besj0},
     {"besj1",  f_besj1},
+    {"besjn",  f_besjn},
     {"besy0",  f_besy0},
     {"besy1",  f_besy1},
+    {"besyn",  f_besyn},
     {"erf",  f_erf},
     {"erfc",  f_erfc},
     {"gamma",  f_gamma},
     {"lgamma",  f_lgamma},
     {"ibeta",  f_ibeta},
     {"voigt",  f_voigt},
-    {"igamma",  f_igamma},
     {"rand",  f_rand},
     {"floor",  f_floor},
     {"ceil",  f_ceil},
-    {"defined", f_exists},	/* DEPRECATED syntax defined(foo) */
 
-    {"norm",  f_normal},	/* XXX-JG */
-    {"inverf",  f_inverse_erf},	/* XXX-JG */
-    {"invnorm",  f_inverse_normal},	/* XXX-JG */
+    {"norm",  f_normal},
+    {"inverf",  f_inverse_erf},
+    {"invnorm",  f_inverse_normal},
+    {"invigamma", f_inverse_igamma},
+    {"invibeta", f_inverse_ibeta},
     {"asinh",  f_asinh},
     {"acosh",  f_acosh},
     {"atanh",  f_atanh},
-    {"lambertw",  f_lambertw}, /* HBB, from G.Kuhnle 20001107 */
-    {"airy",  f_airy},         /* janert, 20090905 */
-    {"expint",  f_expint},     /* Jim Van Zandt, 20101010 */
+    {"lambertw",  f_lambertw},	/* HBB, from G.Kuhnle 20001107 */
+    {"airy",  f_airy},		/* cephes library version */
+#ifdef HAVE_AMOS
+    {"Ai", f_amos_Ai},		/* Amos version from libopenspecfun */
+    {"Bi", f_amos_Bi},		/* Amos version from libopenspecfun */
+    {"BesselI", f_amos_BesselI},/* Amos version from libopenspecfun */
+    {"BesselJ", f_amos_BesselJ},/* Amos version from libopenspecfun */
+    {"BesselK", f_amos_BesselK},/* Amos version from libopenspecfun */
+    {"BesselY", f_amos_BesselY},/* Amos version from libopenspecfun */
+    {"Hankel1", f_Hankel1},	/* Amos version from libopenspecfun */
+    {"Hankel2", f_Hankel2},	/* Amos version from libopenspecfun */
+#endif
+#ifdef HAVE_CEXINT
+    {"expint",  f_amos_cexint},	/* Amos algorithm 683 from libamos */
+#else
+    {"expint",  f_expint},	/* Jim Van Zandt, 20101010 */
+#endif
+
+#ifdef HAVE_COMPLEX_FUNCS
+    {"igamma", f_Igamma},	/* Complex igamma(a,z) */
+    {"LambertW", f_LambertW},	/* Complex W(z,k) */
+    {"lnGamma", f_lnGamma},	/* Complex lnGamma(z) */
+    {"Sign", f_Sign},		/* Complex sign function */
+    {"zeta", f_zeta},		/* Riemann zeta function */
+#else
+    {"igamma",  f_igamma},	/* Jos van der Woude 1992 */
+#endif
+    {"uigamma",  f_uigamma},	/* upper incomplete gamma */
 
 #ifdef HAVE_LIBCERF
     {"cerf", f_cerf},		/* complex error function */
     {"cdawson", f_cdawson},	/* complex Dawson's integral */
     {"erfi", f_erfi},		/* imaginary error function */
     {"VP", f_voigtp},		/* Voigt profile */
+    {"VP_fwhm", f_VP_fwhm},	/* Voigt profile full width at half maximum */
     {"faddeeva", f_faddeeva},	/* Faddeeva rescaled complex error function "w_of_z" */
+    {"FresnelC", f_FresnelC},	/* Fresnel integral cosine term calculated from cerf */
+    {"FresnelS", f_FresnelS},	/* Fresnel integral sine term calculated from cerf */
 #endif
 
-    {"tm_sec",  f_tmsec},	/* for timeseries */
-    {"tm_min",  f_tmmin},	/* for timeseries */
-    {"tm_hour",  f_tmhour},	/* for timeseries */
-    {"tm_mday",  f_tmmday},	/* for timeseries */
-    {"tm_mon",  f_tmmon},	/* for timeseries */
-    {"tm_year",  f_tmyear},	/* for timeseries */
-    {"tm_wday",  f_tmwday},	/* for timeseries */
-    {"tm_yday",  f_tmyday},	/* for timeseries */
+    {"SynchrotronF", f_SynchrotronF},	/* Synchrotron F */
 
+    {"tm_sec",  f_tmsec},	/* time function */
+    {"tm_min",  f_tmmin},	/* time function */
+    {"tm_hour", f_tmhour},	/* time function */
+    {"tm_mday", f_tmmday},	/* time function */
+    {"tm_mon",  f_tmmon},	/* time function */
+    {"tm_year", f_tmyear},	/* time function */
+    {"tm_wday", f_tmwday},	/* time function */
+    {"tm_yday", f_tmyday},	/* time function */
+    {"tm_week", f_tmweek},	/* time function */
+    {"weekdate_iso", f_weekdate_iso},
+    {"weekdate_cdc", f_weekdate_cdc},
+
+    {"join",  f_join},		/* create string from array */
     {"sprintf",  f_sprintf},	/* for string variables only */
     {"gprintf",  f_gprintf},	/* for string variables only */
     {"strlen",  f_strlen},	/* for string variables only */
     {"strstrt",  f_strstrt},	/* for string variables only */
     {"substr",  f_range},	/* for string variables only */
+    {"split", f_split},		/* for string variables only */
+    {"trim",  f_trim},		/* for string variables only */
     {"word",  f_word},		/* for string variables only */
     {"words", f_words},		/* implemented as word(s,-1) */
     {"strftime",  f_strftime},  /* time to string */
@@ -223,8 +284,15 @@ const struct ft_entry GPFAR ft[] =
     {"exist", f_exists},	/* exists("foo") replaces defined(foo) */
     {"exists", f_exists},	/* exists("foo") replaces defined(foo) */
     {"value", f_value},		/* retrieve value of variable known by name */
+    {"index", f_lookup},	/* retrieve index of array entry with known value */
 
     {"hsv2rgb", f_hsv2rgb},	/* color conversion */
+    {"palette", f_palette},	/* palette color lookup */
+    {"rgbcolor", f_rgbcolor},	/* 32bit ARGB color lookup by name or string */
+
+#ifdef VOXEL_GRID_SUPPORT
+    {"voxel", f_voxel},		/* extract value of single voxel */
+#endif
 
     {NULL, NULL}
 };
@@ -238,7 +306,7 @@ static JMP_BUF fpe_env;
 static RETSIGTYPE
 fpe(int an_int)
 {
-#if defined(MSDOS) && !defined(__EMX__) && !defined(DJGPP) && !defined(_Windows)
+#if defined(MSDOS) && !defined(DJGPP)
     /* thanks to lotto@wjh12.UUCP for telling us about this  */
     _fpreset();
 #endif
@@ -264,8 +332,10 @@ real(struct value *val)
 	return ((double) val->v.int_val);
     case CMPLX:
 	return (val->v.cmplx_val.real);
-    case STRING:              /* is this ever used? */
+    case STRING:
 	return (atof(val->v.string_val));
+    case NOTDEFINED:
+	return not_a_number();
     default:
 	int_error(NO_CARET, "unknown type in real()");
     }
@@ -274,25 +344,6 @@ real(struct value *val)
 }
 
 
-/* returns the real part of val, converted to int if necessary */
-int
-real_int(struct value *val)
-{
-    switch (val->type) {
-    case INTGR:
-	return val->v.int_val;
-    case CMPLX:
-	return (int) val->v.cmplx_val.real;
-    case STRING:
-	return atoi(val->v.string_val);
-    default:
-	int_error(NO_CARET, "unknown type in real_int()");
-    }
-    /* NOTREACHED */
-    return 0;
-}
-
-
 /* returns the imag part of val */
 double
 imag(struct value *val)
@@ -307,6 +358,8 @@ imag(struct value *val)
 	/*     x = 2;  plot sprintf(format,x)         */
 	int_warn(NO_CARET, "encountered a string when expecting a number");
 	int_error(NO_CARET, "Did you try to generate a file name using dummy variable x or y?");
+    case NOTDEFINED:
+	return not_a_number();
     default:
 	int_error(NO_CARET, "unknown type in imag()");
     }
@@ -322,7 +375,7 @@ magnitude(struct value *val)
 {
     switch (val->type) {
     case INTGR:
-	return ((double) abs(val->v.int_val));
+	return (fabs((double)val->v.int_val));
     case CMPLX:
 	{
 	    /* The straightforward implementation sqrt(r*r+i*i)
@@ -390,7 +443,7 @@ Gcomplex(struct value *a, double realpart, double imagpart)
 
 
 struct value *
-Ginteger(struct value *a, int i)
+Ginteger(struct value *a, intgr_t i)
 {
     a->type = INTGR;
     a->v.int_val = i;
@@ -401,24 +454,88 @@ struct value *
 Gstring(struct value *a, char *s)
 {
     a->type = STRING;
-    a->v.string_val = s;
+    a->v.string_val = s ? s : strdup("");
     return (a);
 }
 
-/* It is always safe to call gpfree_string with a->type is INTGR or CMPLX.
- * However it would be fatal to call it with a->type = STRING if a->string_val
- * was not obtained by a previous call to gp_alloc(), or has already been freed.
- * Thus 'a->type' is set to INTGR afterwards to make subsequent calls safe.
+/* The rationale for introducing this routine was that multiple call sites
+ * wanted to write a new value to a variable that might already have one.
+ * free_value() was intended to consider all possible previous value types
+ * and free attached memory for types that had any.
+ *
+ * Caveat:  When freeing values popped from the evaluation stack,
+ * datablocks and permanent arrays must not be freed because these are
+ * calls by reference to a continuing global variable.
+ * So the caller must clear the type field before calling free_value.
  */
-struct value *
+void
+free_value(struct value *a)
+{
+    switch (a->type) {
+	case INTGR:
+	case CMPLX:
+			break;
+	case STRING:
+			gpfree_string(a);
+			break;
+	case ARRAY:
+			gpfree_array(a);
+			break;
+	case DATABLOCK:
+			gpfree_datablock(a);
+			break;
+	case FUNCTIONBLOCK:
+			gpfree_functionblock(a);
+			break;
+	case VOXELGRID: /* Should not happen! */
+	default:	/* INVALID_VALUE INVALID_NAME */
+			break;
+    }
+    a->type = NOTDEFINED;
+}
+
+/* It would be fatal to call gpfree_string with a->type = STRING if
+ * a->string_val has already been freed.
+ * Setting 'a->type' to NOTDEFINED makes subsequent calls safe.
+ */
+void
 gpfree_string(struct value *a)
 {
     if (a->type == STRING) {
 	free(a->v.string_val);
-	/* I would have set it to INVALID if such a type existed */
-	a->type = INTGR;
+	a->type = NOTDEFINED;
     }
-    return a;
+}
+
+void
+gpfree_array(struct value *a)
+{
+    int i;
+    int size;
+
+    if (a->type == ARRAY) {
+	size = a->v.value_array[0].v.int_val;
+	for (i=1; i<=size; i++)
+	    gpfree_string(&(a->v.value_array[i]));
+	free(a->v.value_array);
+	a->type = NOTDEFINED;
+    }
+}
+
+
+void
+init_array( struct udvt_entry *array, int size )
+{
+    struct value *A;
+    int i;
+
+    free_value(&array->udv_value);
+    array->udv_value.v.value_array = gp_alloc((size+1) * sizeof(t_value), "init_array");
+    array->udv_value.type = ARRAY;
+    A = array->udv_value.v.value_array;
+    A[0].v.int_val = size;
+    for (i = 0; i <= size; i++)
+	A[i].type = NOTDEFINED;
 }
 
 /* some machines have trouble with exp(-x) for large x
@@ -477,34 +594,49 @@ pop(struct value *x)
 /*
  * Allow autoconversion of string variables to floats if they
  * are dereferenced in a numeric context.
+ * Jun 2022: Stricter error checking for non-numeric string.
  */
 struct value *
 pop_or_convert_from_string(struct value *v)
 {
-    (void) pop(v);
+    pop(v);
 
-    /* DEBUG Dec 2014 - Consolidate sanity check for variable type */
     /* FIXME: Test for INVALID_VALUE? Other corner cases? */
     if (v->type == INVALID_NAME)
 	int_error(NO_CARET, "invalid dummy variable name");
 
     if (v->type == STRING) {
-	char *eov;
+	char *string = v->v.string_val;
+	char *eov = string;
+	char trailing = *eov;
 
-	if (strspn(v->v.string_val,"0123456789 ") == strlen(v->v.string_val)) {
-	    int i = atoi(v->v.string_val);
-	    gpfree_string(v);
-	    Ginteger(v, i);
-	} else {
-	    double d = strtod(v->v.string_val,&eov);
-	    if (v->v.string_val == eov) {
-		gpfree_string(v);
-		int_error(NO_CARET,"Non-numeric string found where a numeric expression was expected");
-	    }
-	    gpfree_string(v);
-	    Gcomplex(v, d, 0.);
-	    FPRINTF((stderr,"converted string to CMPLX value %g\n",real(v)));
+	/* If the string contains no decimal point, try to interpret it as an integer.
+	 * We treat a string starting with "0x" as a hexadecimal; everything else
+	 * as decimal.  So int("010") promotes to 10, not 8.
+	 */
+	if (strcspn(string, ".") == strlen(string)) {
+	    long long li;
+	    if (string[0] == '0' && string[1] == 'x')
+		li = strtoll( string, &eov, 16 );
+	    else
+		li = strtoll( string, &eov, 10 );
+	    trailing = *eov;
+	    Ginteger(v, li);
 	}
+	/* Successful interpretation as an integer leaves (eov != string).
+	 * Otherwise try again as a floating point, including oddball cases like
+	 * "NaN" or "-Inf" that contain no decimal point.
+	 */
+	if (eov == string) {
+	    double d = strtod(string, &eov);
+	    trailing = *eov;
+	    Gcomplex(v, d, 0.);
+	}
+	free(string);	/* NB: invalidates dereference of eov */
+	if (eov == string)
+	    int_error(NO_CARET,"Non-numeric string found where a numeric expression was expected");
+	if (trailing && !isspace(trailing))
+	    int_warn(NO_CARET,"Trailing characters after numeric expression");
     }
     return(v);
 }
@@ -515,6 +647,7 @@ push(struct value *x)
     if (s_p == STACK_DEPTH - 1)
 	int_error(NO_CARET, "stack overflow");
     stack[++s_p] = *x;
+
     /* WARNING - This is a memory leak if the string is not later freed */
     if (x->type == STRING && x->v.string_val)
 	stack[s_p].v.string_val = gp_strdup(x->v.string_val);
@@ -627,15 +760,26 @@ execute_at(struct at_type *at_ptr)
     jump_offset = saved_jump_offset;
 }
 
-/* May 2013: Old hackery #ifdef'ed out so that input of Inf/NaN */
-/* values through evaluation is treated equivalently to direct  */
-/* input of a formated value.  See revised imageNaN demo.       */
+/* As of May 2013 input of Inf/NaN values through evaluation is treated */
+/* equivalently to direct input of a formatted value.  See imageNaN.dem. */
 void
 evaluate_at(struct at_type *at_ptr, struct value *val_ptr)
 {
+    /* A test for if (undefined) is allowed only immediately following
+     * evalute_at() or eval_link_function().  Both must clear it on entry
+     * so that the value on return reflects what really happened.
+     */
     undefined = FALSE;
+    val_ptr->type = NOTDEFINED;
+
     errno = 0;
-    reset_stack();
+
+    /* Normally the stack is cleared prior to each and every expression
+     * evaluation.   However doing so during execution of a function block
+     * can make the stack invalid when the function block exits.
+     */
+    if (!evaluate_inside_functionblock)
+	reset_stack();
 
     if (!evaluate_inside_using || !df_nofpe_trap) {
 	if (SETJMP(fpe_env, 1))
@@ -650,61 +794,45 @@ evaluate_at(struct at_type *at_ptr, struct value *val_ptr)
 
     if (errno == EDOM || errno == ERANGE)
 	undefined = TRUE;
-#if (1)	/* New code */
-    else if (!undefined) {
-	(void) pop(val_ptr);
-	check_stack();
-    }
 
-#else /* Old hackery */
-    else if (!undefined) { /* undefined (but not errno) may have been set by matherr */
-	(void) pop(val_ptr);
-	check_stack();
-	/* At least one machine (ATT 3b1) computes Inf without a SIGFPE */
-	if (val_ptr->type != STRING) {
-	    double temp = real(val_ptr);
-	    if (temp > VERYLARGE || temp < -VERYLARGE)
-		undefined = TRUE;
-	}
-    }
-#if defined(NeXT) || defined(ultrix)
-    /*
-     * linux was able to fit curves which NeXT gave up on -- traced it to
-     * silently returning NaN for the undefined cases and plowing ahead
-     * I can force that behavior this way.  (0.0/0.0 generates NaN)
+    /* Pop value even if it is undefined.
+     * That seems preferable to leaving garbage on the stack.
      */
-    if (undefined && (errno == EDOM || errno == ERANGE)) {	/* corey@cac */
-	undefined = FALSE;
-	errno = 0;
-	Gcomplex(val_ptr, 0.0 / 0.0, 0.0 / 0.0);
-    }
-#endif /* NeXT || ultrix */
-#endif /* old hackery */
+    if (s_p >= 0)
+	pop(val_ptr);
+    if (!evaluate_inside_functionblock)
+	check_stack();
 }
 
 void
-free_at(struct at_type *at_ptr)
+free_action_entry(struct at_entry *a)
+{
+    /* if union a->arg is used as a->arg.v_arg free potential string */
+    if ( a->index == PUSHC || a->index == DOLLARS )
+	gpfree_string(&(a->arg.v_arg));
+    /* a summation contains its own action table wrapped in a private udf */
+    if (a->index == SUM) {
+	real_free_at(a->arg.udf_arg->at);
+	free(a->arg.udf_arg);
+    }
+#ifdef HAVE_EXTERNAL_FUNCTIONS
+    /* external function calls contain a parameter list */
+    if (a->index == CALLE)
+	free(a->arg.exf_arg);
+#endif
+}
+
+void
+real_free_at(struct at_type *at_ptr)
 {
     int i;
     /* All string constants belonging to this action table have to be
      * freed before destruction. */
     if (!at_ptr)
         return;
-    for(i=0; i<at_ptr->a_count; i++) {
+    for (i=0; i<at_ptr->a_count; i++) {
 	struct at_entry *a = &(at_ptr->actions[i]);
-	/* if union a->arg is used as a->arg.v_arg free potential string */
-	if ( a->index == PUSHC || a->index == DOLLARS )
-	    gpfree_string(&(a->arg.v_arg));
-	/* a summation contains its own action table wrapped in a private udf */
-	if (a->index == SUM) {
-	    free_at(a->arg.udf_arg->at);
-	    free(a->arg.udf_arg);
-	}
-#ifdef HAVE_EXTERNAL_FUNCTIONS
-	/* external function calls contain a parameter list */
-	if (a->index == CALLE)
-	    free(a->arg.exf_arg);
-#endif
+	free_action_entry(a);
     }
     free(at_ptr);
 }
@@ -729,8 +857,7 @@ add_udv_by_name(char *key)
 	gp_alloc(sizeof(struct udvt_entry), "value");
     (*udv_ptr)->next_udv = NULL;
     (*udv_ptr)->udv_name = gp_strdup(key);
-    (*udv_ptr)->udv_undef = TRUE;
-    (*udv_ptr)->udv_value.type = 0;
+    (*udv_ptr)->udv_value.type = NOTDEFINED;
     return (*udv_ptr);
 }
 
@@ -759,20 +886,30 @@ del_udv_by_name(char *key, TBOOLEAN wildcard)
 	/* Forbidden to delete GPVAL_* */
 	if (!strncmp(udv_ptr->udv_name,"GPVAL",5))
 	    ;
+	else if (!strncmp(udv_ptr->udv_name,"GNUTERM",7))
+	    ;
 
  	/* exact match */
 	else if (!wildcard && !strcmp(key, udv_ptr->udv_name)) {
-	    udv_ptr->udv_undef = TRUE;
-	    gpfree_string(&(udv_ptr->udv_value));
-	    gpfree_datablock(&(udv_ptr->udv_value));
+	    if (called_from(udv_ptr->udv_name)) {
+		FPRINTF((stderr, "cannot self-delete %s", udv_ptr->udv_name));
+		break;
+	    }
+	    gpfree_vgrid(udv_ptr);
+	    free_value(&(udv_ptr->udv_value));
+	    udv_ptr->udv_value.type = NOTDEFINED;
 	    break;
 	}
 
 	/* wildcard match: prefix matches */
 	else if ( wildcard && !strncmp(key, udv_ptr->udv_name, strlen(key)) ) {
-	    udv_ptr->udv_undef = TRUE;
-	    gpfree_string(&(udv_ptr->udv_value));
-	    gpfree_datablock(&(udv_ptr->udv_value));
+	    if (called_from(udv_ptr->udv_name)) {
+		FPRINTF((stderr, "cannot self-delete %s", udv_ptr->udv_name));
+		break;
+	    }
+	    gpfree_vgrid(udv_ptr);
+	    free_value(&(udv_ptr->udv_value));
+	    udv_ptr->udv_value.type = NOTDEFINED;
 	    /* no break - keep looking! */
 	}
 
@@ -780,6 +917,20 @@ del_udv_by_name(char *key, TBOOLEAN wildcard)
     }
 }
 
+#ifdef USE_WATCHPOINTS
+struct udft_entry *
+get_udf_by_token(int t_num)
+{
+    struct udft_entry **udf_ptr = &first_udf;
+    while (*udf_ptr) {
+	if (equals(t_num, (*udf_ptr)->udf_name))
+	    return *udf_ptr;
+	udf_ptr = &((*udf_ptr)->next_udf);
+    }
+    return NULL;
+}
+#endif
+
 /* Clear (delete) all user defined functions */
 void
 clear_udf_list()
@@ -798,12 +949,13 @@ clear_udf_list()
     first_udf = NULL;
 }
 
-static void update_plot_bounds __PROTO((void));
-static void fill_gpval_axis __PROTO((AXIS_INDEX axis));
-static void set_gpval_axis_sth_double __PROTO((const char *prefix, AXIS_INDEX axis, const char *suffix, double value, int is_int));
+static void update_plot_bounds(void);
+static void fill_gpval_axis(AXIS_INDEX axis);
+static void fill_gpval_sysinfo(void);
+static void set_gpval_axis_sth_double(const char *prefix, AXIS_INDEX axis, const char *suffix, double value);
 
 static void
-set_gpval_axis_sth_double(const char *prefix, AXIS_INDEX axis, const char *suffix, double value, int is_int)
+set_gpval_axis_sth_double(const char *prefix, AXIS_INDEX axis, const char *suffix, double value)
 {
     struct udvt_entry *v;
     char *cc, s[24];
@@ -811,13 +963,9 @@ set_gpval_axis_sth_double(const char *prefix, AXIS_INDEX axis, const char *suffi
     for (cc=s; *cc; cc++)
 	*cc = toupper((unsigned char)*cc); /* make the name uppercase */
     v = add_udv_by_name(s);
-    if (!v) 
+    if (!v)
 	return; /* should not happen */
-    v->udv_undef = FALSE;
-    if (is_int)
-	Ginteger(&v->udv_value, (int)(value+0.5));
-    else
-	Gcomplex(&v->udv_value, value, 0);
+    Gcomplex(&v->udv_value, value, 0);
 }
 
 static void
@@ -825,15 +973,13 @@ fill_gpval_axis(AXIS_INDEX axis)
 {
     const char *prefix = "GPVAL";
     AXIS *ap = &axis_array[axis];
-    double a = AXIS_DE_LOG_VALUE(axis, ap->min);
-    double b = AXIS_DE_LOG_VALUE(axis, ap->max);
-    set_gpval_axis_sth_double(prefix, axis, "MIN", a, 0);
-    set_gpval_axis_sth_double(prefix, axis, "MAX", b, 0);
-    set_gpval_axis_sth_double(prefix, axis, "LOG", ap->base, 0);
+    set_gpval_axis_sth_double(prefix, axis, "MIN", ap->min);
+    set_gpval_axis_sth_double(prefix, axis, "MAX", ap->max);
+    set_gpval_axis_sth_double(prefix, axis, "LOG", ap->base);
 
     if (axis < POLAR_AXIS) {
-	set_gpval_axis_sth_double("GPVAL_DATA", axis, "MIN", AXIS_DE_LOG_VALUE(axis, ap->data_min), 0);
-	set_gpval_axis_sth_double("GPVAL_DATA", axis, "MAX", AXIS_DE_LOG_VALUE(axis, ap->data_max), 0);
+	set_gpval_axis_sth_double("GPVAL_DATA", axis, "MIN", ap->data_min);
+	set_gpval_axis_sth_double("GPVAL_DATA", axis, "MAX", ap->data_max);
     }
 }
 
@@ -846,22 +992,19 @@ fill_gpval_string(char *var, const char *stringvalue)
     struct udvt_entry *v = add_udv_by_name(var);
     if (!v)
 	return;
-    if (v->udv_undef == FALSE && !strcmp(v->udv_value.v.string_val, stringvalue))
+    if (v->udv_value.type == STRING && !strcmp(v->udv_value.v.string_val, stringvalue))
 	return;
-    if (v->udv_undef)
-	v->udv_undef = FALSE;
     else
 	gpfree_string(&v->udv_value);
     Gstring(&v->udv_value, gp_strdup(stringvalue));
 }
 
 void
-fill_gpval_integer(char *var, int value)
+fill_gpval_integer(char *var, intgr_t value)
 {
     struct udvt_entry *v = add_udv_by_name(var);
     if (!v)
 	return;
-    v->udv_undef = FALSE;
     Ginteger(&v->udv_value, value);
 }
 
@@ -871,7 +1014,6 @@ fill_gpval_float(char *var, double value)
     struct udvt_entry *v = add_udv_by_name(var);
     if (!v)
 	return;
-    v->udv_undef = FALSE;
     Gcomplex(&v->udv_value, value, 0);
 }
 
@@ -881,7 +1023,6 @@ fill_gpval_complex(char *var, double areal, double aimag)
     struct udvt_entry *v = add_udv_by_name(var);
     if (!v)
 	return;
-    v->udv_undef = FALSE;
     Gcomplex(&v->udv_value, areal, aimag);
 }
 
@@ -892,13 +1033,16 @@ fill_gpval_complex(char *var, double areal, double aimag)
 static void
 update_plot_bounds(void)
 {
-    fill_gpval_integer("GPVAL_TERM_XMIN", axis_array[FIRST_X_AXIS].term_lower / term->tscale);
-    fill_gpval_integer("GPVAL_TERM_XMAX", axis_array[FIRST_X_AXIS].term_upper / term->tscale);
-    fill_gpval_integer("GPVAL_TERM_YMIN", axis_array[FIRST_Y_AXIS].term_lower / term->tscale);
-    fill_gpval_integer("GPVAL_TERM_YMAX", axis_array[FIRST_Y_AXIS].term_upper / term->tscale);
+    fill_gpval_float("GPVAL_TERM_XMIN", (double)axis_array[FIRST_X_AXIS].term_lower / term->tscale);
+    fill_gpval_float("GPVAL_TERM_XMAX", (double)axis_array[FIRST_X_AXIS].term_upper / term->tscale);
+    fill_gpval_float("GPVAL_TERM_YMIN", (double)axis_array[FIRST_Y_AXIS].term_lower / term->tscale);
+    fill_gpval_float("GPVAL_TERM_YMAX", (double)axis_array[FIRST_Y_AXIS].term_upper / term->tscale);
     fill_gpval_integer("GPVAL_TERM_XSIZE", canvas.xright+1);
     fill_gpval_integer("GPVAL_TERM_YSIZE", canvas.ytop+1);
     fill_gpval_integer("GPVAL_TERM_SCALE", term->tscale);
+    /* May be useful for debugging font problems */
+    fill_gpval_integer("GPVAL_TERM_HCHAR", term->h_char);
+    fill_gpval_integer("GPVAL_TERM_VCHAR", term->v_char);
 }
 
 /*
@@ -937,6 +1081,17 @@ update_gpval_variables(int context)
 	fill_gpval_float("GPVAL_VIEW_ROT_Z", surface_rot_z);
 	fill_gpval_float("GPVAL_VIEW_SCALE", surface_scale);
 	fill_gpval_float("GPVAL_VIEW_ZSCALE", surface_zscale);
+	fill_gpval_float("GPVAL_VIEW_AZIMUTH", azimuth);
+
+	/* Screen coordinates of 3D rotational center and radius of the sphere */
+	/* in which x/y axes are drawn after 'set view equal xy[z]' */
+	fill_gpval_float("GPVAL_VIEW_XCENT",
+		(double)(canvas.xright+1 - xmiddle)/(double)(canvas.xright+1));
+	fill_gpval_float("GPVAL_VIEW_YCENT",
+		1.0 - (double)(canvas.ytop+1 - ymiddle)/(double)(canvas.ytop+1));
+	fill_gpval_float("GPVAL_VIEW_RADIUS",
+		0.5 * surface_scale * xscaler/(double)(canvas.xright+1));
+
 	return;
     }
 
@@ -953,6 +1108,9 @@ update_gpval_variables(int context)
 	fill_gpval_string("GPVAL_TERMOPTIONS", term_options);
 	fill_gpval_string("GPVAL_OUTPUT", (outstr) ? outstr : "");
 	fill_gpval_string("GPVAL_ENCODING", encoding_names[encoding]);
+	fill_gpval_string("GPVAL_MINUS_SIGN", minus_sign ? minus_sign : "-");
+	fill_gpval_string("GPVAL_MICRO", micro ? micro : "u");
+	fill_gpval_string("GPVAL_DEGREE_SIGN", degree_sign);
     }
 
     /* If we are called from int_error() then set the error state */
@@ -963,15 +1121,13 @@ update_gpval_variables(int context)
     if (context == 3) {
 	struct udvt_entry *v = add_udv_by_name("GPVAL_VERSION");
 	char *tmp;
-	if (v && v->udv_undef == TRUE) {
-	    v->udv_undef = FALSE;
+	if (v && v->udv_value.type == NOTDEFINED)
 	    Gcomplex(&v->udv_value, atof(gnuplot_version), 0);
-	}
 	v = add_udv_by_name("GPVAL_PATCHLEVEL");
-	if (v && v->udv_undef == TRUE)
+	if (v && v->udv_value.type == NOTDEFINED)
 	    fill_gpval_string("GPVAL_PATCHLEVEL", gnuplot_patchlevel);
 	v = add_udv_by_name("GPVAL_COMPILE_OPTIONS");
-	if (v && v->udv_undef == TRUE)
+	if (v && v->udv_value.type == NOTDEFINED)
 	    fill_gpval_string("GPVAL_COMPILE_OPTIONS", compile_options);
 
 	/* Start-up values */
@@ -988,21 +1144,27 @@ update_gpval_variables(int context)
 	/* Permanent copy of user-clobberable variables pi and NaN */
 	fill_gpval_float("GPVAL_pi", M_PI);
 	fill_gpval_float("GPVAL_NaN", not_a_number());
+
+	/* System information */
+	fill_gpval_sysinfo();
     }
 
     if (context == 3 || context == 4) {
 	fill_gpval_integer("GPVAL_ERRNO", 0);
 	fill_gpval_string("GPVAL_ERRMSG","");
+	fill_gpval_integer("GPVAL_SYSTEM_ERRNO", 0);
+	fill_gpval_string("GPVAL_SYSTEM_ERRMSG","");
     }
 
+    /* GPVAL_PWD is unreliable.  If the current directory becomes invalid,
+     * GPVAL_PWD does not reflect this.  If this matters, the user can
+     * instead do something like    MY_PWD = "`pwd`"
+     */
     if (context == 3 || context == 5) {
-	char *save_file = NULL;
-	save_file = (char *) gp_alloc(PATH_MAX, "filling GPVAL_PWD");
-	if (save_file) {
-	    GP_GETCWD(save_file, PATH_MAX);
-	    fill_gpval_string("GPVAL_PWD", save_file);
-	    free(save_file);
-	}
+	char *save_file = gp_alloc(PATH_MAX, "GPVAL_PWD");
+	int ierror = (GP_GETCWD(save_file, PATH_MAX) == NULL);
+	fill_gpval_string("GPVAL_PWD", ierror ? "" : save_file);
+	free(save_file);
     }
 
     if (context == 6) {
@@ -1010,6 +1172,58 @@ update_gpval_variables(int context)
     }
 }
 
+/* System information is stored in GPVAL_BITS GPVAL_MACHINE GPVAL_SYSNAME */
+#ifdef HAVE_UNAME
+# include <sys/utsname.h>
+#elif defined(_WIN32)
+# include <windows.h>
+#endif
+
+void
+fill_gpval_sysinfo()
+{
+/* For linux/posix systems with uname */
+#ifdef HAVE_UNAME
+    struct utsname uts;
+
+    if (uname(&uts) < 0)
+	return;
+    fill_gpval_string("GPVAL_SYSNAME", uts.sysname);
+    fill_gpval_string("GPVAL_MACHINE", uts.machine);
+
+/* For Windows systems */
+#elif defined(_WIN32)
+    SYSTEM_INFO stInfo;
+    OSVERSIONINFO osvi;
+    char s[30];
+
+    ZeroMemory(&osvi, sizeof(OSVERSIONINFO));
+    osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
+    GetVersionEx(&osvi);
+    snprintf(s, 30, "Windows_NT-%ld.%ld", osvi.dwMajorVersion, osvi.dwMinorVersion);
+    fill_gpval_string("GPVAL_SYSNAME", s);
+
+    GetSystemInfo(&stInfo);
+    switch (stInfo.wProcessorArchitecture)
+    {
+    case PROCESSOR_ARCHITECTURE_INTEL:
+        fill_gpval_string("GPVAL_MACHINE", "x86");
+        break;
+    case PROCESSOR_ARCHITECTURE_IA64:
+        fill_gpval_string("GPVAL_MACHINE", "ia64");
+       break;
+    case PROCESSOR_ARCHITECTURE_AMD64:
+        fill_gpval_string("GPVAL_MACHINE", "x86_64");
+        break;
+    default:
+        fill_gpval_string("GPVAL_MACHINE", "unknown");
+    }
+#endif
+
+/* For all systems */
+    fill_gpval_integer("GPVAL_BITS", 8 * sizeof(void *));
+}
+
 /* Callable wrapper for the words() internal function */
 int
 gp_words(char *string)
@@ -1030,33 +1244,74 @@ gp_word(char *string, int i)
     struct value a;
 
     push(Gstring(&a, string));
-    push(Ginteger(&a, i));
+    push(Ginteger(&a, (intgr_t)i));
     f_word((union argument *)NULL);
     pop(&a);
 
     return a.v.string_val;
 }
 
-
-/* Evaluate the function linking secondary axis to primary axis */
-double
-eval_link_function(int axis, double raw_coord)
+/* New (version 5.5)
+ * The evaluation stack can now return an ARRAY value, but in order to do so
+ * without memory leaks or corruption it must make sure that the allocated
+ * content is distinct from the original content.
+ * I.e. {array A = ["foo"]; B = A; A = 0;} must leave a valid copy of "foo" in B[1].
+ */
+void
+make_array_permanent(struct value *array)
 {
-    udft_entry *link_udf = axis_array[axis].link_udf;
-    int dummy_var;
-    struct value a;
+    struct value *copy;
+    int i, size;
 
-    if (axis == FIRST_Y_AXIS || axis == SECOND_Y_AXIS)
-	dummy_var = 1;
-    else
-	dummy_var = 0;
-    link_udf->dummy_values[1-dummy_var].type = INVALID_NAME;
+    /* If this array was generated by the evaluation stack it is safe to use
+     * but the temporary flag must be cleared.
+     */
+    if (array->v.value_array[0].type == TEMP_ARRAY) {
+	array->v.value_array[0].type = NOTDEFINED;
+	return;
+    }
 
-    Gcomplex(&link_udf->dummy_values[dummy_var], raw_coord, 0.0);
-    evaluate_at(link_udf->at, &a);
+    /* If it was a pre-existing array (no temporary flag) then we must make
+     * a clean copy of the whole thing.
+     */
+    size = array->v.value_array[0].v.int_val;
+    copy = gp_alloc( (size+1) * sizeof(struct value), "array copy");
+    memcpy( copy, array->v.value_array, (size+1) * sizeof(struct value) );
+    for (i=0; i<= size; i++) {
+	if (copy[i].type == STRING)
+	    copy[i].v.string_val = strdup(copy[i].v.string_val);
+    }
+    copy[0].type = NOTDEFINED;
+    array->v.value_array = copy;
+}
 
-    if (a.type != CMPLX)
-	a = udv_NaN->udv_value;
+/* Extract a portion of the array full[N] into a new array slice[M]
+ * where M <= N
+ */
+struct value *
+array_slice( struct value *full, int beg, int end)
+{
+    struct value *array = full->v.value_array;
+    struct value *slice;
+    int i,j;
 
-    return a.v.cmplx_val.real;
+    /* Sanity checks */
+    if (beg < 1)
+	beg = 1;
+    if (end > array[0].v.int_val)
+	end = array[0].v.int_val;
+    if (end < beg)
+	beg = 1, end = 0;
+
+    slice = gp_alloc( (2 + end - beg) * sizeof(struct value), "array slice" );
+    slice[0].type = TEMP_ARRAY;
+    slice[0].v.int_val = 1 + end - beg;
+
+    for (i = beg, j = 1; i <= end; i++,j++) {
+	slice[j] = array[i];
+	if (slice[j].type == STRING)
+	    slice[j].v.string_val = strdup(slice[j].v.string_val);
+    }
+
+    return slice;
 }
